@@ -11,11 +11,24 @@ globalVariables(c("Snapshot.ID.Tag", "Snapshot.Time.Stamp", "Time.after.Planting
                   "Weight.Before", "Weight.After", "Water.Amount", "r", "Cumulative.Propn"),
                 "imageData", add = TRUE)
 
-#Function to move RGB_SV1, RB_SV2, RGB_TV prefix to a suffix without the RBG_ 
-"pre2suffix" <- function(name)
-{ prefix <- (strsplit(name, ".", fixed=TRUE))[[1]][1]
-  if (prefix == "RGB_SV1" | prefix == "RGB_SV2" | prefix == "RGB_TV")
-  { suffix <- (strsplit(prefix, "_", fixed=TRUE))[[1]][2]
+#Function to move camera prefix to a suffix without the characters up to the first _ (eg RBG_) 
+#If no _ then moves whole prefix.
+#Assumes that prefix ends at first full.stop
+"pre2suffix" <- function(name, cameraNames, keepCameraType)
+{ 
+  prefix <- (strsplit(name, ".", fixed=TRUE))[[1]][1]
+  if (any(cameraNames %in% prefix))
+  { 
+    if (grepl("_", prefix) && !keepCameraType)
+    {
+      suffix <- (strsplit(prefix, "_"))[[1]]
+      if (length(suffix) == 2)
+        suffix <- suffix[2]
+      else
+        suffix <- paste(suffix[2:length(suffix)], collapse = "_")
+    }
+    else
+      suffix <- prefix
     fst <- nchar(prefix)+2
     name <- paste(substring(name, first=fst), suffix, sep=".")
   }
@@ -25,7 +38,8 @@ globalVariables(c("Snapshot.ID.Tag", "Snapshot.Time.Stamp", "Time.after.Planting
 "importExcel" <- function(file, sheet="raw data", sep = ",", cartId = "Snapshot.ID.Tag", 
                           imageTimes = "Snapshot.Time.Stamp", 
                           timeAfterStart = "Time.after.Planting..d.", 
-                          prefix2suffix = TRUE, 
+                          cameraNames = NULL, cameraType = "RGB", 
+                          prefix2suffix = TRUE, keepCameraType = FALSE, 
                           startTime = NULL, timeFormat = "%Y-%m-%d %H:%M", 
                           imagetimesPlot = TRUE, ...)
 { 
@@ -50,14 +64,47 @@ globalVariables(c("Snapshot.ID.Tag", "Snapshot.Time.Stamp", "Time.after.Planting
   else
     stop("File name does not include csv or xlsx")
   ncinput <- ncol(raw.dat)
+  if (!(cartId %in% names(raw.dat)))
+    stop("cartId not in imported data")
+  if (!(imageTimes%in% names(raw.dat)))
+    stop("imageTimes not in imported data")
   
-  #Rename columns to move prefix for camera View to become a suffix without the RGB_
+
+  #Rename image columns 
+  #Change cameraNames as sprecified by camera names
+  vars <- names(raw.dat)
+  if (!is.null(names(cameraNames)))
+  {
+    old.names <- names(cameraNames)
+    for (old in old.names)
+      vars <- gsub(old, cameraNames[old], vars, fixed = TRUE)
+    names(raw.dat) <- vars
+  } else #find
+  {
+    cameraNames <- vars[grep(cameraType, vars, fixed = TRUE)]
+    if (length(cameraNames) == 0)
+      warning(paste("No imaging variables for a camera of type ", cameraType, " found", sep=""))
+    cameraNames <- strsplit(cameraNames, ".", fixed=TRUE)
+    cameraNames <- unique(unlist(lapply(cameraNames, 
+                                        function(name) {return(name[[1]][1])})))
+    names(cameraNames) <- cameraNames
+  }
+  
+  #Move prefix for camera View to become a suffix without the RGB_
   if (prefix2suffix)
   { 
-    vars <- names(raw.dat)
-    newvars <- sapply(vars[1:length(vars)], pre2suffix)
+    newvars <- sapply(vars[1:length(vars)], pre2suffix, 
+                      cameraNames = cameraNames, keepCameraType = keepCameraType)
     names(newvars) <- NULL
     names(raw.dat)[match(vars, names(raw.dat))] <- newvars
+  } else
+  {
+    if (!keepCameraType)
+    {
+      vars <- names(raw.dat)
+      vars <- gsub(paste(cameraType, "_",sep = ""), "", vars, fixed = TRUE) 
+      names(raw.dat) <- vars
+    }
   }
   
   #Change day calculation to take away a time origin and truncate to the nearest whole day
@@ -69,7 +116,7 @@ globalVariables(c("Snapshot.ID.Tag", "Snapshot.Time.Stamp", "Time.after.Planting
   #Plot the imaging times if required
   if (imagetimesPlot)
     imagetimesPlot(raw.dat, intervals=timeAfterStart, timePositions = "Hour", 
-                   ...)
+                   groupVariable = cartId, ...)
 
   #Check unique for Snapshot.ID.Tag, Time.after.Planting..d.
   combs <- as.vector(table(raw.dat[[cartId]], raw.dat[[timeAfterStart]]))
@@ -89,29 +136,39 @@ globalVariables(c("Snapshot.ID.Tag", "Snapshot.Time.Stamp", "Time.after.Planting
 "longitudinalPrime" <- function(data, cartId = "Snapshot.ID.Tag", 
                                 imageTimes = "Snapshot.Time.Stamp", 
                                 timeAfterStart = "Time.after.Planting..d.", 
-                                idcolumns = c("Genotype.ID","Treatment.1"), 
+                                idcolumns = c("Genotype.ID","Treatment.1"),
+                                traits = NULL,
+                                cameras = NULL, 
                                 smarthouse.lev = NULL, 
-                                calcWaterLoss = TRUE, pixelsPERcm = 18)
-{ #Extract variables from data to form data frame of longitudinal data
+                                calcWaterLoss = TRUE, pixelsPERcm = 1)
+{ 
+  #Extract variables from data to form data frame of longitudinal data
   posndatevars <- c(cartId,timeAfterStart,
                     "Smarthouse","Lane","Position",imageTimes)
+  if (is.null(traits))
+    traits <- list(all = c("Area", "Boundary.Points.To.Area.Ratio", "Caliper.Length",
+                           "Compactness", "Convex.Hull.Area"), 
+                   side = c("Center.Of.Mass.Y", "Max.Dist.Above.Horizon.Line"))
+  if (is.null(cameras))
+    cameras <- list(all = c("SV1", "SV2", "TV"),
+                    side = c("SV1", "SV2"))
+  if (length(cameras) == 0)
+    imagevars <- unlist(traits)
+  else
+    imagevars <- unlist(mapply(FUN =  function(traits, names) {
+                                        as.vector(t(outer(traits, names, paste, sep = ".")))
+                                      }, 
+                               traits, cameras))
+  names(imagevars) <- NULL
   imagevars <- c("Weight.Before","Weight.After","Water.Amount",
-                 "Projected.Shoot.Area..pixels.",
-                 "Area.SV1", "Boundary.Points.To.Area.Ratio.SV1", "Caliper.Length.SV1",
-                 "Center.Of.Mass.Y.SV1", "Compactness.SV1", "Convex.Hull.Area.SV1", 
-                 "Max.Dist.Above.Horizon.Line.SV1", 
-                 "Area.SV2", "Boundary.Points.To.Area.Ratio.SV2", "Caliper.Length.SV2", 
-                 "Center.Of.Mass.Y.SV2", "Compactness.SV2", "Convex.Hull.Area.SV2", 
-                 "Max.Dist.Above.Horizon.Line.SV2", 
-                 "Area.TV", "Boundary.Points.To.Area.Ratio.TV",  "Caliper.Length.TV", 
-                 "Compactness.TV", "Convex.Hull.Area.TV")
+                 "Projected.Shoot.Area..pixels.", imagevars)
   vars <- c(posndatevars, idcolumns, imagevars)
   
   #Check that vars are in data
-  if (any(is.na(match(vars, names(data)))))
-  { miss <- vars[is.na(match(vars, names(data)))]
+  if (!all(vars %in% names(data)))
+  { 
     stop(paste("The following variables are not present in data:  ",
-               paste(miss, collapse = ", "), sep = ""))
+               paste(vars[!(vars %in% names(data))], collapse = ", "), sep = ""))
   }
   
   longi.prime.dat <- data[, vars]
@@ -214,8 +271,9 @@ return(longi.prime.dat)
 #Function to add design factors for blocked, split plot design
 "designFactors" <- function(data, insertName = NULL, designfactorMethod = "LanePosition", 
                             nzones = 6, nlanesperzone = 4, nmainplotsperlane = 11, nsubplotspermain = 2)
-{ options <- c("LanePosition","StandardOrder")
-  desfactor <- options[unlist(lapply(designfactorMethod, check.arg.values, options=options))]
+{ 
+  options <- c("LanePosition","StandardOrder")
+  desfactor <- options[check.arg.values(designfactorMethod, options=options)]
 
   #Extract variables from data
   vars <- names(data)
@@ -324,8 +382,8 @@ return(longi.prime.dat)
 "splitContGRdiff" <- function(data, responses, INDICES, 
                               which.rates = c("AGR","PGR","RGR"), suffices.rates=NULL, 
                               times.factor = "Days")
-{ options <- c("AGR","PGR","RGR")
-
+{ 
+  options <- c("AGR","PGR","RGR")
   opt <- options[unlist(lapply(which.rates, check.arg.values, options=options))]
   if (!is.null(suffices.rates) & length(opt) != length(suffices.rates))
     stop("The length of of which.rates and suffices.rates should be equal")
@@ -1188,16 +1246,56 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
   return(facs)
 }
 
+plotDeviations <- function(dat, x, obs, smoothed, devnplots = "none", x.title = NULL, 
+                           facet.x = "Treatment.1", facet.y = "Smarthouse", 
+                           labeller = NULL, df)  
+{
+  ggfacet <- list()
+  #Set up facet if have any
+  if (facet.x != "." | facet.y != ".")
+  {
+    facet.string <- paste(facet.y,facet.x,sep="~")
+    if (is.null(labeller))
+      ggfacet <- list(facet_grid(facet.string))
+    else
+      ggfacet <- list(facet_grid(facet.string, labeller = labeller))
+  }
+  
+  if ("absolute" %in% devnplots)
+  {
+    dat$deviations <- dat[[obs]] - dat[[smoothed]]
+    print(ggplot(data = dat, aes_string(x=x, y = "deviations")) +
+            theme_bw() + 
+            geom_boxplot() + 
+            geom_hline(yintercept=0, colour="blue") +
+            ylab(paste("Absolute", obs, "deviations for df", df, sep = " ")) + 
+            xlab(x.title) + ggfacet)
+  }
+  if ("relative" %in% devnplots)
+  {
+    dat$deviations <- (dat[[obs]] - dat[[smoothed]])/dat[[smoothed]]
+    print(ggplot(data = dat, aes_string(x=x, y = "deviations")) +
+            theme_bw() + 
+            geom_boxplot() + 
+            geom_hline(yintercept=0, colour="blue") +
+            ylab(paste("Relative", obs, "deviations for df", df, sep = " ")) + 
+            xlab(x.title) + ggfacet)
+  }
+  invisible()
+}
 
 "probeDF" <- function(data, response = "Area", xname="xDays", 
                       individuals="Snapshot.ID.Tag", 
                       na.x.action="exclude", na.y.action = "exclude", 
                       df, correctBoundaries = FALSE, 
                       get.rates = TRUE, rates.method="differences", 
-                      times.factor = "Days", x = NULL, 
+                      times.factor = "Days", x = NULL, x.title = NULL, 
                       facet.x = "Treatment.1", facet.y = "Smarthouse", 
+                      labeller = NULL, 
                       which.traits = c("response", "AGR", "RGR"), 
-                      which.plots = "smoothedonly", ggplotFuncs = NULL, ...)
+                      which.plots = "smoothedonly",
+                      deviations.boxplots = "none", 
+                      ggplotFuncs = NULL, ...)
 { 
   #check input arguments
   impArgs <- match.call()
@@ -1205,12 +1303,19 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
     stop("na.rm has been deprecated; use na.x.action and na.y.action")
   options <- c("differences","derivative")
   opt <- options[check.arg.values(rates.method, options=options)]
-  options <- c("none", "smoothedonly", "bothseparately", "compare")
+  options <- c("none", "smoothedonly", "bothseparately", "compare", "deviationsboxplot")
   plots <- options[check.arg.values(which.plots, options=options)]
   if (any(c("bothseparately", "compare") %in% plots))
     plotunsmooth <- TRUE
   else
     plotunsmooth <- FALSE
+  options <- c("none", "absolute", "relative")
+  devnplots <- options[unlist(lapply(deviations.boxplots, check.arg.values, 
+                                     options=options))]
+  if ("none" %in% devnplots & length(devnplots) > 1)
+    devnplots <- "none"
+  if (is.null(x.title))
+    x.title <- times.factor
   options <- c("response", "AGR", "RGR", "all")
   traits <- options[unlist(lapply(which.traits, check.arg.values, options=options))]
   if ("all" %in% traits)
@@ -1240,7 +1345,7 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
   #Smooth response and form growth rates
   response.smooth <- paste(response, "smooth", sep=".")
   responses.smooth <- response.smooth 
-  if (plotunsmooth & get.rates)
+  if ((plotunsmooth | !("none" %in% devnplots)) & get.rates)
   {
     tmp <- splitContGRdiff(tmp, response, INDICES=individuals,
                            which.rates = c("AGR","RGR"), times.factor=times.factor)
@@ -1279,15 +1384,15 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
   }
   
   #Plot some combination of unsmoothed and smoothed response, AGR and RGR
-  if (!("none" %in% plots))
+  if (!("none" %in% plots) | !("none" %in% devnplots))
   { 
     responses.tmp <- names(tmp)
     #Plot response
     if (plotunsmooth & "response" %in% traits)
     { 
       pltu <- longiPlot(data = tmp, x=x, response = response, individuals = individuals, 
-                        facet.x=facet.x, facet.y=facet.y, 
-                        title="Unsmoothed response", y.title = response, 
+                        facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                        title="Unsmoothed response", x.title = x.title, y.title = response, 
                         printPlot=FALSE, ...)
       if (!is.null(ggplotFuncs))
         for (f in ggplotFuncs)
@@ -1296,11 +1401,12 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
           print(pltu)
     }
     if ("compare" %in% plots)
-      for (r in paste(response.smooth, df, sep="."))
+      for (degfree in df)
       { 
+        r <- paste(response.smooth, degfree, sep=".")
         plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                         facet.x=facet.x, facet.y=facet.y, 
-                         title="Smoothed response", y.title = r, 
+                         facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                         title="Smoothed response", x.title = x.title, y.title = r, 
                          printPlot=FALSE, ...)
         if (!is.null(ggplotFuncs))
           for (f in ggplotFuncs)
@@ -1309,19 +1415,30 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
           pushViewport(viewport(layout = grid.layout(1, 2)))
           print(pltu, vp=viewport(layout.pos.row=1, layout.pos.col=1))
           print(plt, vp=viewport(layout.pos.row=1, layout.pos.col=2))
-      }
-    else
-      for (r in paste(response.smooth, df, sep="."))
-      { 
-        plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                         facet.x=facet.x, facet.y=facet.y, 
-                         title="Smoothed response", y.title = r, 
-                         printPlot=FALSE, ...)
-        if (!is.null(ggplotFuncs))
-          for (f in ggplotFuncs)
-            plt <- plt + f
-          print(plt)
-      }
+          plotDeviations(dat = tmp, x = times.factor, obs = response, smoothed = r, 
+                         devnplots = devnplots, x.title = x.title, 
+                         facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                         df = degfree)
+      } else
+        if ("compare" %in% plots)
+          for (degfree in df)
+          { 
+            r <- paste(response.smooth, degfree, sep=".")
+            plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
+                             facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                             title="Smoothed response", x.title = x.title, y.title = r, 
+                             printPlot=FALSE, ...)
+            if (!is.null(ggplotFuncs))
+            {
+              for (f in ggplotFuncs)
+                plt <- plt + f
+            }
+            print(plt)
+            plotDeviations(dat = tmp, x = times.factor, obs = response, smoothed = r, 
+                           devnplots = devnplots, x.title = x.title, 
+                           facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                           df = degfree)
+          }
     
     #Plot AGRs
     if ("AGR" %in% traits)
@@ -1329,9 +1446,10 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
       if (plotunsmooth & "AGR" %in% traits)
       { 
         pltu <- longiPlot(data = tmp, x=x, response = paste(response,"AGR",sep="."), 
-                          individuals = individuals, facet.x=facet.x, facet.y=facet.y, 
+                          individuals = individuals, 
+                          facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
                           title="Unsmoothed AGR by difference", 
-                          y.title = paste(response,"AGR",sep="."), 
+                          x.title = x.title, y.title = paste(response,"AGR",sep="."), 
                           printPlot=FALSE, ...)
         if (!is.null(ggplotFuncs))
           for (f in ggplotFuncs)
@@ -1339,13 +1457,13 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
           if (!("compare" %in% plots))
             print(pltu)
       }
-      responses.plt <- paste(response.smooth, "AGR", df, sep=".")
       if ("compare" %in% plots)
-        for (r in responses.plt)
+        for (degfree in df)
         { 
+          r <- paste(response.smooth, "AGR", degfree, sep=".")
           plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                           facet.x=facet.x, facet.y=facet.y, 
-                           title="Smoothed AGR", y.title = r, 
+                           facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                           title="Smoothed AGR", x.title = x.title, y.title = r, 
                            printPlot=FALSE, ...)
           if (!is.null(ggplotFuncs))
             for (f in ggplotFuncs)
@@ -1354,30 +1472,41 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
             pushViewport(viewport(layout = grid.layout(1, 2)))
             print(pltu, vp=viewport(layout.pos.row=1, layout.pos.col=1))
             print(plt, vp=viewport(layout.pos.row=1, layout.pos.col=2))
-        }
-      else
-        for (r in responses.plt)
-        { 
-          plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                           facet.x=facet.x, facet.y=facet.y, 
-                           title=paste("Smoothed AGR by ", opt, sep=""), 
-                           y.title = r, printPlot=FALSE, ...)
-          if (!is.null(ggplotFuncs))
-            for (f in ggplotFuncs)
-              plt <- plt + f
-            print(plt)
-        }
+            plotDeviations(dat = tmp, x = times.factor, 
+                           obs = paste(response,"AGR",sep="."), smoothed = r, 
+                           devnplots = devnplots, x.title = x.title, 
+                           facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                           df = degfree)
+        } else
+          for (degfree in df)
+          { 
+            r <- paste(response.smooth, "AGR", degfree, sep=".")
+            plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
+                             facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                             title=paste("Smoothed AGR by ", opt, sep=""), 
+                             x.title = x.title, y.title = r, printPlot=FALSE, ...)
+            if (!is.null(ggplotFuncs))
+              for (f in ggplotFuncs)
+                plt <- plt + f
+              print(plt)
+              plotDeviations(dat = tmp, x = times.factor, 
+                             obs = paste(response,"AGR",sep="."), smoothed = r, 
+                             devnplots = devnplots, x.title = x.title, 
+                             facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                             df = degfree)
+          }
     }
-    
+      
     #Plot RGR
     if ("RGR" %in% traits)
     { 
       if (plotunsmooth & "RGR" %in% traits)
       { 
         pltu <- longiPlot(data = tmp, x=x, response = paste(response,"RGR",sep="."), 
-                          individuals = individuals, facet.x=facet.x, facet.y=facet.y, 
+                          individuals = individuals, 
+                          facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
                           title="Unsmoothed RGR by difference", 
-                          y.title = paste(response,"RGR",sep="."), 
+                          x.title = x.title, y.title = paste(response,"RGR",sep="."), 
                           printPlot=FALSE, ...)
         if (!is.null(ggplotFuncs))
           for (f in ggplotFuncs)
@@ -1385,13 +1514,13 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
           if (!("compare" %in% plots))
             print(pltu)
       }
-      responses.plt <- paste(response.smooth, "RGR", df, sep=".")
       if ("compare" %in% plots)
-        for (r in responses.plt)
+        for (degfree in df)
         { 
+          r <- paste(response.smooth, "RGR", degfree, sep=".")
           plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                           facet.x=facet.x, facet.y=facet.y, 
-                           title="Smoothed RGR", y.title = r, 
+                           facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                           title="Smoothed RGR", x.title = x.title, y.title = r, 
                            printPlot=FALSE, ...)
           if (!is.null(ggplotFuncs))
             for (f in ggplotFuncs)
@@ -1400,19 +1529,29 @@ predict.ncsSpline <- function(object, x, correctBoundaries = FALSE,
             pushViewport(viewport(layout = grid.layout(1, 2)))
             print(pltu, vp=viewport(layout.pos.row=1, layout.pos.col=1))
             print(plt, vp=viewport(layout.pos.row=1, layout.pos.col=2))
-        }
-      else
-        for (r in responses.plt)
-        { 
-          plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
-                           facet.x=facet.x, facet.y=facet.y, 
-                           title=paste("Smoothed RGR by ", opt, sep=""), 
-                           y.title = r, printPlot=FALSE, ...)
-          if (!is.null(ggplotFuncs))
-            for (f in ggplotFuncs)
-              plt <- plt + f
-            print(plt)
-        }
+            plotDeviations(dat = tmp, x = times.factor, 
+                           obs = paste(response,"RGR",sep="."), smoothed = r, 
+                           devnplots = devnplots, x.title = x.title, 
+                           facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                           df = degfree)
+        } else
+          for (degfree in df)
+          { 
+            r <- paste(response.smooth, "RGR", degfree, sep=".")
+            plt <- longiPlot(data = tmp, x=x, response = r, individuals = individuals, 
+                             facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                             title=paste("Smoothed RGR by ", opt, sep=""), 
+                             x.title = x.title, y.title = r, printPlot=FALSE, ...)
+            if (!is.null(ggplotFuncs))
+              for (f in ggplotFuncs)
+                plt <- plt + f
+              print(plt)
+              plotDeviations(dat = tmp, x = times.factor, 
+                             obs = paste(response,"RGR",sep="."), smoothed = r, 
+                             devnplots = devnplots, x.title = x.title, 
+                             facet.x=facet.x, facet.y=facet.y, labeller = labeller, 
+                             df = degfree)
+          }
     }
   }
   invisible(tmp)
